@@ -44,7 +44,10 @@ pub use veci::Veci as Vecx;
 use lazy_static::lazy_static;
 use ruc::*;
 use std::{
-    env, ptr,
+    convert::TryFrom,
+    env, fmt,
+    mem::size_of,
+    ptr,
     sync::atomic::{AtomicBool, Ordering},
 };
 
@@ -105,6 +108,42 @@ pub fn flush_data() {
     });
 }
 
+/// numberic key
+pub trait NumKey: Clone + Copy + PartialEq + Eq + PartialOrd + Ord + fmt::Debug {
+    /// key => bytes
+    fn to_bytes(&self) -> Vec<u8>;
+    /// bytes => key
+    fn from_bytes(b: &[u8]) -> Result<Self>;
+}
+
+macro_rules! impl_nk_trait {
+    ($t: ty) => {
+        impl NumKey for $t {
+            fn to_bytes(&self) -> Vec<u8> {
+                self.to_le_bytes().to_vec()
+            }
+            fn from_bytes(b: &[u8]) -> Result<Self> {
+                <[u8; size_of::<$t>()]>::try_from(b)
+                    .c(d!())
+                    .map(<$t>::from_le_bytes)
+            }
+        }
+    };
+}
+
+impl_nk_trait!(i8);
+impl_nk_trait!(i16);
+impl_nk_trait!(i32);
+impl_nk_trait!(i64);
+impl_nk_trait!(i128);
+impl_nk_trait!(isize);
+impl_nk_trait!(u8);
+impl_nk_trait!(u16);
+impl_nk_trait!(u32);
+impl_nk_trait!(u64);
+impl_nk_trait!(u128);
+impl_nk_trait!(usize);
+
 /// Try once more when we fail to open a db.
 #[macro_export]
 macro_rules! try_twice {
